@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import sys
+import textwrap
 from pathlib import Path
 
 import pkg_resources
@@ -9,7 +10,7 @@ from PySide6.QtGui import QCursor, QMovie
 from PySide6.QtUiTools import QUiLoader
 from PySide6.QtWidgets import QButtonGroup, QMainWindow, QFileDialog, QDialog
 
-from plagdef.gui.model import ResultsTableModel
+from plagdef.gui.model import ResultsTableModel, DocumentPairMatches
 
 UI_FILES = {
     'main_window': pkg_resources.resource_filename(__name__, 'ui/main_window.ui'),
@@ -18,6 +19,7 @@ UI_FILES = {
     'error_widget': pkg_resources.resource_filename(__name__, 'ui/error_widget.ui'),
     'no_results_widget': pkg_resources.resource_filename(__name__, 'ui/no_results_widget.ui'),
     'results_widget': pkg_resources.resource_filename(__name__, 'ui/results_widget.ui'),
+    'matches_dialog': pkg_resources.resource_filename(__name__, 'ui/matches_dialog.ui'),
 }
 
 
@@ -71,10 +73,12 @@ class HomeView(View):
         self.widget.sel_lang_label.setVisible(False)
         self.widget.change_lang_button.setCursor(QCursor(Qt.PointingHandCursor))
         self.widget.change_lang_button.setVisible(False)
+        self.widget.exclude_common_check_box.setVisible(False)
 
-    def register_for_signals(self, open_folder=None, remove_folder=None, select_lang=None,
+    def register_for_signals(self, open_folder=None, remove_folder=None, exclude_common=None, select_lang=None,
                              reset_lang_sel=None, detect=None):
         self.widget.open_folder_button.clicked.connect(lambda: open_folder())
+        self.widget.exclude_common_check_box.toggled.connect(lambda checked: exclude_common(checked))
         self.widget.remove_folder_button.clicked.connect(lambda: remove_folder())
         self.widget.lang_button_group.buttonClicked.connect(lambda: select_lang())
         self.widget.change_lang_button.clicked.connect(lambda: reset_lang_sel())
@@ -84,12 +88,17 @@ class HomeView(View):
         self.widget.folder_name_label.setText(
             f'<html><head/><body><p align="center"><span style="font-size:12pt; color:#ffffff;"> '
             f'{folder_name}</span></p></body></html>')
+        self.widget.folder_name_label.setWordWrap(True)
         self.widget.open_folder_button.setVisible(False)
         self.widget.folder_name_label.setVisible(True)
         self.widget.recursive_check_box.setVisible(True)
         self.widget.remove_folder_button.setVisible(True)
         [radio.setEnabled(True) for radio in self.widget.lang_button_group.buttons()]
 
+    def common_folder_selected(self, folder_name: str):
+        wrapped_name = textwrap.fill(folder_name, 20)
+        self.widget.exclude_common_check_box.setText(f'Excluding text in:\n{wrapped_name}')
+
     def reset_folder_selection(self):
         self.recursive = False
         self.widget.recursive_check_box.setChecked(False)
@@ -112,6 +121,7 @@ class HomeView(View):
         self.widget.sel_lang_label.setVisible(True)
         self.widget.change_lang_button.setVisible(True)
         self.widget.detect_button.setEnabled(True)
+        self.widget.exclude_common_check_box.setVisible(True)
 
     def reset_language_selection(self):
         self.lang = None
@@ -119,12 +129,18 @@ class HomeView(View):
         self.widget.lang_button_group.checkedButton().setChecked(False)
         self.widget.lang_button_group.setExclusive(True)
         self.widget.detect_button.setEnabled(False)
+        self.widget.exclude_common_check_box.setVisible(False)
+        self.reset_exclude_common()
         self.widget.sel_lang_label.setVisible(False)
         self.widget.change_lang_button.setVisible(False)
         [radio.setVisible(True) for radio in self.widget.lang_button_group.buttons()]
         self.widget.remove_folder_button.setVisible(True)
         self.widget.recursive_check_box.setEnabled(True)
 
+    def reset_exclude_common(self):
+        self.widget.exclude_common_check_box.setChecked(False)
+        self.widget.exclude_common_check_box.setText('Exclude common text')
+
     def on_destroy(self):
         self.reset_language_selection()
         [radio.setEnabled(False) for radio in self.widget.lang_button_group.buttons()]
@@ -198,13 +214,56 @@ class ResultView(View):
         self.widget.again_button_res.setCursor(QCursor(Qt.PointingHandCursor))
 
     def on_init(self, data=None):
+        self.widget.doc_pairs_label.setText(f"Found {len(data)} suspicious document pair"
+                                            f"{'s' if len(data) > 1 else ''}.")
         model = ResultsTableModel(data)
         self.widget.results_table.setModel(model)
         self.widget.results_table.resizeRowsToContents()
         self.widget.results_table.resizeColumnsToContents()
 
-    def register_for_signals(self, again=None):
+    def register_for_signals(self, again=None, select_pair=None):
         self.widget.again_button_res.clicked.connect(lambda: again())
+        self.widget.results_table.doubleClicked.connect(
+            lambda index: select_pair(index.model().doc_pair_matches(index)))
+
+
+class MatchesDialog:
+    def __init__(self):
+        self.widget = _load_ui_file(Path(UI_FILES['matches_dialog']))
+        self._doc_pair_matches = None
+        self._selected = 0
+
+    def open(self, doc_pair_matches: DocumentPairMatches):
+        self._doc_pair_matches = doc_pair_matches
+        self.widget.doc1_label.setText(self._doc_pair_matches.doc1.name)
+        self.widget.doc2_label.setText(self._doc_pair_matches.doc2.name)
+        self._selected = 0
+        self._show_match()
+        self.widget.exec_()
+
+    def _show_match(self):
+        doc1, doc2 = self._doc_pair_matches.doc1, self._doc_pair_matches.doc2
+        self.widget.doc1_text.setText(self._doc_pair_matches.matches[self._selected].frag_from_doc(doc1).text)
+        self.widget.doc2_text.setText(self._doc_pair_matches.matches[self._selected].frag_from_doc(doc2).text)
+        self.widget.page_label.setText(f'{self._selected + 1}/{len(self._doc_pair_matches)}')
+        self.widget.prev_button.setVisible(True)
+        self.widget.next_button.setVisible(True)
+        if self._selected == 0:
+            self.widget.prev_button.setVisible(False)
+        if self._selected + 1 == len(self._doc_pair_matches):
+            self.widget.next_button.setVisible(False)
+
+    def register_for_signals(self, prev_match=None, next_match=None):
+        self.widget.prev_button.clicked.connect(lambda: prev_match())
+        self.widget.next_button.clicked.connect(lambda: next_match())
+
+    def prev_match(self):
+        self._selected -= 1
+        self._show_match()
+
+    def next_match(self):
+        self._selected += 1
+        self._show_match()
 
 
 def _load_ui_file(path: Path) -> QMainWindow:
