@@ -1,7 +1,6 @@
 from __future__ import annotations
 
 import logging
-import os
 from ast import literal_eval
 from configparser import ConfigParser
 from itertools import islice
@@ -35,8 +34,9 @@ class DocumentFileRepository:
 
     def list(self) -> set[Document]:
         files = list(self._list_files())
-        return set(thread_map(self._read_file, files, max_workers=os.cpu_count(),
-                              desc=f"Reading documents in '{self._dir_path}'", unit='doc'))
+        docs = thread_map(self._read_file, files, desc=f"Reading documents in '{self._dir_path}'",
+                          unit='doc', total=len(files))
+        return set(filter(None, docs))
 
     def _read_file(self, file):
         if file.suffix == '.pdf':
